package com.qmth.distributed.print.business.util;

import com.alibaba.fastjson.JSON;
import com.qmth.distributed.print.business.bean.marking.ConvertJpgStorage;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.teachcloud.common.bean.vo.FilePathVo;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.PageSizeEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.service.BasicAttachmentService;
import com.qmth.teachcloud.common.service.FileUploadService;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.teachcloud.common.util.HtmlToPdfUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.entity.ContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Lazy;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Component;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.StringJoiner;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description: 题卡html转化为
 * @Author: CaoZixuan
 * @Date: 2022-04-14
 */
@Component
public class HtmlToJpgUtil {

    @Lazy
    @Resource
    private PrintCommonService printCommonService;
    @Resource
    private DictionaryConfig dictionaryConfig;
    @Resource
    private BasicAttachmentService basicAttachmentService;
    @Resource
    private FileUploadService fileUploadService;

    private final static Logger log = LoggerFactory.getLogger(HtmlToJpgUtil.class);

    /**
     * 转换html -> jpg : 生成本地临时文件
     *
     * @param id           题卡Id
     * @param htmlContent  html内容
     * @param pageSizeEnum 转化pdf的纸张类型
     */
    public List<BasicAttachment> convertHtmlToJpg(Long id, String htmlContent, PageSizeEnum pageSizeEnum) {
        boolean oss = dictionaryConfig.sysDomain().isOss();
        StringJoiner stringJoiner = new StringJoiner("");
        if (!oss && Objects.nonNull(dictionaryConfig.fssPublicDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPublicDomain().getConfig()) && !dictionaryConfig.fssPublicDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
            stringJoiner.add(dictionaryConfig.fssPublicDomain().getConfig()).add(File.separator);
        }
        stringJoiner = SystemConstant.getDirName(stringJoiner, UploadFileEnum.IMAGE, true);

        // 本地存储目录
        String rootPath = stringJoiner.toString();
        // html临时路径
        String htmlPath = rootPath + File.separator + "html" + File.separator + id + SystemConstant.HTML_PREFIX;
        // pdf临时路径
        String pdfPath = rootPath + File.separator + "pdf" + File.separator + id + SystemConstant.PDF_PREFIX;
        // jpg临时路径
        String jpgPath = rootPath + File.separator + "jpg" + File.separator + id;
        List<BasicAttachment> basicAttachmentList = new ArrayList<>();
        // html
        try {
            File htmlFile = new File(htmlPath);
            if (!htmlFile.exists()) {
                htmlFile.getParentFile().mkdirs();
            }
            // 生成html文件
            FileCopyUtils.copy(htmlContent.getBytes(StandardCharsets.UTF_8), htmlFile);
            // 转pdf文件
            HtmlToPdfUtil.convert(htmlPath, pdfPath, pageSizeEnum);
            // pdf文件转jpg
            List<File> jpgFileList = ConvertUtil.pdfToImageFile(pdfPath, jpgPath);

            for (File jpgFile : jpgFileList) {
                InputStream inputStream = new FileInputStream(jpgFile);
                MultipartFile multipartFile = new MockMultipartFile(jpgFile.getName(), jpgFile.getName(),
                        ContentType.APPLICATION_OCTET_STREAM.toString(), inputStream);
                BasicAttachment basicAttachment = printCommonService.saveAttachment(multipartFile, DigestUtils.md5Hex(multipartFile.getBytes()), UploadFileEnum.FILE);
                basicAttachmentList.add(basicAttachment);
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            ConvertUtil.delFolder(rootPath);
        }
        return basicAttachmentList;
    }

    public List<ConvertJpgStorage> convertPdfToJpg(Long cardId, File file, Long userId) {
        // jpg临时路径
        List<ConvertJpgStorage> convertJpgStorages = new ArrayList<>();
        List<BasicAttachment> basicAttachmentList = new ArrayList<>();
        // html
        try {
            // pdf文件转jpg
            List<File> jpgFileList = ConvertUtil.pdfToImageFile(file);
            for (int i = 0; i < jpgFileList.size(); i++) {
                int index = i + 1;
                String fileName = cardId + "-" + index + SystemConstant.JPG_PREFIX;
                FilePathVo filePathVo = fileUploadService.uploadFile(jpgFileList.get(i), UploadFileEnum.IMAGE, fileName);
                BasicAttachment basicAttachment = new BasicAttachment(JSON.toJSONString(filePathVo), fileName, SystemConstant.PDF_PREFIX, new BigDecimal(file.length()), filePathVo.getMd5(), userId);
                basicAttachmentService.save(basicAttachment);
                basicAttachmentList.add(basicAttachment);

                convertJpgStorages.add(new ConvertJpgStorage(basicAttachment.getId(), index));
            }
        } catch (Exception e) {
            basicAttachmentService.batchDeleteAttachment(basicAttachmentList);
        }
        return convertJpgStorages;
    }

    /**
     * 转换html -> jpg : 生成本地临时文件
     *
     * @param id           题卡名称
     * @param htmlContent  题卡内容
     * @param pageSizeEnum 转化pdf的纸张类型
     */
    public List<ConvertJpgStorage> convertHtmlToJpgByCard(Long id, String htmlContent, PageSizeEnum pageSizeEnum) {
        List<ConvertJpgStorage> convertJpgStorageList = new ArrayList<>();
        try {
            String regex = "\\$\\{\\w+}";
            htmlContent = htmlContent.replaceAll("<img src=\"data:image/png;base64,\\$\\{examNumber\\}\">", "");
            htmlContent = htmlContent.replaceAll(regex, "");
            convertJpgStorageList = this.convertHtmlToJpg(id, htmlContent, pageSizeEnum).stream().flatMap(e -> {
                ConvertJpgStorage tmp = new ConvertJpgStorage();
                String name = e.getName();
                String index = name.substring(name.lastIndexOf(SystemConstant.HYPHEN) + 1);
                tmp.setAttachmentId(e.getId());
                tmp.setIndex(Integer.valueOf(index));
                return Stream.of(tmp);
            }).collect(Collectors.toList());

            if (convertJpgStorageList.size() == 0) {
                throw ExceptionResultEnum.ERROR.exception("生成jpg文件异常");
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return convertJpgStorageList;
    }
}
